@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2004 Wesley J. Landaker <wjl@icecavern.net> 
+ * Copyright (C) 2001 major mms <http://www.geocities.com/majormms/>
  *
  * MiMMS is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,28 +17,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  */
 
-/* 
- * Copyright (C) 2000-2001 major mms <http://www.geocities.com/majormms/>
- *
- * This file is part of libmms
- * 
- * libmms is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- * 
- * libmms is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * utility functions to handle communication with an mms server
- */
-
 /*
  * mms://netshow.msn.com/msnbc8 
  * mms://216.106.172.144/bbc1099/ads/ibeam/0_ibeamEarth_aaab00020_15_350k.asf
@@ -45,18 +24,27 @@
  * mms://193.159.244.12/n24_wmt_mid
  */
 
+#define MIMMS_VERSION "MiMMS v0.0.9 ($Date: 2004/10/22 14:37:47 $)\n"
+
 #include <stdint.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <netdb.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
 #include <fcntl.h>
+#include <unistd.h>
 #include <uuid/uuid.h>
-#include "bswap.h"
-#include "gettext.h"
+#include <popt.h>
+#include <regex.h>
+#include <time.h>
+
+#include <string>
+using std::string;
+
+#define _(x) x
 
 #ifdef DEBUG
 # define dprintf(x...) printf(x)
@@ -78,7 +66,6 @@ int num_stream_ids;
 int stream_ids[20];
 int output_fh;
 
-
 static void put_32 (command_t *cmd, uint32_t value) {
   for (int i=0; i<4; i++) {
     cmd->buf[cmd->num_bytes++] = (value >> (8*i)) & 0xff;
@@ -165,7 +152,7 @@ static void send_command (int s, int command, uint32_t switches,
 
 }
 
-static void string_utf16(char *dest, char *src, int len) {
+static void string_utf16(char *dest, const char *src, int len) {
   int i;
 
   memset (dest, 0, 1000);
@@ -236,6 +223,11 @@ static int get_data (int s, char *buf, size_t count) {
 
     len = recv (s, &buf[total], count-total, 0);
 
+    if (len>0) {
+       dprintf ("[%d/%d]", total, count);
+       fflush (stdout);
+    }
+
     if (len<0) {
       perror ("read error:");
       return 0;
@@ -243,14 +235,11 @@ static int get_data (int s, char *buf, size_t count) {
 
     total += len;
 
-    if (len != 0) {
-      dprintf ("[%d/%d]", total, count);
-      fflush (stdout);
-    }
+    if (len == 0) return total;
 
   }
 
-  return 1;
+  return total;
 
 }
 
@@ -422,7 +411,7 @@ static int get_media_packet (int s, int padding) {
     dprintf (_("pre_header[%d] = %02x (%d)\n"),
 	    i, pre_header[i], pre_header[i]);
 
-  if (pre_header[4] == 0x04) {
+  if (pre_header[4] == 0x04 || pre_header[4] == 0x44 || pre_header[4] == 0xd1 ) {
 
     int packet_len;
 
@@ -487,7 +476,7 @@ static int get_media_packet (int s, int padding) {
   return 1;
 }
 
-int main (int argc, char **argv) {
+int main (int argc, const char **argv) {
 
   int                  s ;
   struct sockaddr_in   sa;
@@ -497,47 +486,146 @@ int main (int argc, char **argv) {
   uint8_t              asf_header[8192];
   int                  asf_header_len;
   int                  len, i, packet_length;
-  char                 host[256];
-  char                *path, *url, *file, *cp;
+  const char          *host;
+  int                  port;
+  const char          *path;
+  const char          *file;
+  int                  time = -1;
 
-  if (argc != 2 || 
-      (argc == 1 &&
-       (!strncmp("--h", argv[1], 3) ||
-	!strncmp("-h", argv[1], 2))) ||
-      (argc == 2 && strncmp("mms://", argv[1], 6))
-      ) {
-    fprintf (stderr,_("usage: %s mms://<host>[:port]\n"), argv[0]);
+  char *paramfile = 0;
+  int paramtime = 0;
+  int paramversion = 0;
+
+  time_t end_time = -1;  
+
+  // parse command line arguments
+  // host = hostname
+  // path = full path
+  // file = output filename
+
+  poptOption paramtable[] = {
+    { 
+      "output", 'o', POPT_ARG_STRING, &paramfile, 0,
+      "Write the output to the given file instead of the name in the URL.", 
+      "<file>" 
+    },
+    { 
+      "time", 't', POPT_ARG_INT, &paramtime, 0,
+      "Record only for given number of minutes (useful for live streams).", 
+      "<minutes>" 
+    },
+    POPT_AUTOHELP
+    { 
+      "version", 0, POPT_ARG_NONE, &paramversion, 0,
+      "Show program version information.",
+      0
+    },
+    POPT_TABLEEND
+  };
+
+  poptContext paramcontext = 
+    poptGetContext(0, argc, argv, paramtable, 0);
+
+  poptSetOtherOptionHelp(paramcontext, "[options] mms://<host>[:port]/<path>");
+  
+  if (argc < 2) {
+    poptPrintUsage(paramcontext, stderr, 0);
     exit(1);
   }
 
-  /* parse url */
+  int paramresult = poptGetNextOpt(paramcontext);
+
+  if (paramversion) {
+    fprintf(stderr, MIMMS_VERSION);
+    exit(1);
+  }
+
+  if (paramresult != -1) {
+    fprintf(stderr, "(%d) %s: %s\n", paramresult,
+	    poptBadOption(paramcontext, POPT_BADOPTION_NOALIAS),
+	    poptStrerror(paramresult));
+    poptPrintUsage(paramcontext, stderr, 0);
+    exit(1);
+  }
+
+  const char **mms_url = poptGetArgs(paramcontext);
+  if (!mms_url) {
+    poptPrintUsage(paramcontext, stderr, 0);
+    exit(1);    
+  }
+
+  regex_t mms_regex;
+  regmatch_t mms_regmatch[5];
+  {
+    int result = regcomp(&mms_regex, 
+	  "^(mms)://([-.[:alnum:]]+)(:[[:digit:]]+)?(/.*)$",
+	  REG_EXTENDED);
+    if (result) {
+      fprintf(stderr,"BUG: error compiling mms_regex\n");
+      exit(2);
+    }
+  }
   
-  url = argv[1];
-  strncpy (host, &url[6], 255);
-  cp = strchr(host,'/');
-  if (cp) *cp= 0;
+  {
+    int result = regexec(&mms_regex, mms_url[0], 5, mms_regmatch, 0);
+    if (result) {
+      fprintf(stderr, "Unparseable url = '%s'\n", mms_url[0]);
+      poptPrintUsage(paramcontext, stderr, 0);
+      exit(1);
+    }
+  }
+
+  string protocol = string(&mms_url[0][mms_regmatch[1].rm_so],
+			   &mms_url[0][mms_regmatch[1].rm_eo]);
+  if (protocol != "mms") {
+    fprintf(stderr, "Unknown protocol = '%s'", protocol.c_str());
+    poptPrintUsage(paramcontext, stderr, 0);
+    exit (1);
+  }
+
+  string host_string = string(&mms_url[0][mms_regmatch[2].rm_so],
+			      &mms_url[0][mms_regmatch[2].rm_eo]);
+  host = host_string.c_str();
+
+  port = 1755;
+  if (mms_regmatch[3].rm_so != -1) {
+    string port_string = string(&mms_url[0][mms_regmatch[3].rm_so+1],
+				&mms_url[0][mms_regmatch[3].rm_eo]);
+    port = atoi(port_string.c_str());
+  }
+
+  string path_string = string(&mms_url[0][mms_regmatch[4].rm_so],
+			      &mms_url[0][mms_regmatch[4].rm_eo]);
+  path = path_string.c_str() + 1;
+  file = strrchr(path, '/');
+  if (file == 0) {
+    file = path;
+  } else {
+    file += 1;
+  }
+
+  if (paramtime) time = paramtime;
+  if (paramfile) file = paramfile;
 
   fprintf (stderr,_("host : >%s<\n"), host);
-
-  path = strchr(&url[6], '/');
-  if (!path) path = "/invalid";
-  path += 1;
-
+  fprintf (stderr,_("port : >%d<\n"), port);
   fprintf (stderr,_("path : >%s<\n"), path);
-
-  file = strrchr (url, '/');
-  if (!file) file = "invalid";
-
   fprintf (stderr,_("file : >%s<\n"), file);
+  fprintf (stderr,_("time : >%d<\n"), time);
 
-  output_fh = open (&file[1], O_WRONLY | O_CREAT | O_TRUNC, 0644);
+  if (time != -1) {
+    ::time(&end_time);
+    end_time += time*60;
+  }
+
+  output_fh = open (file, O_WRONLY | O_CREAT | O_TRUNC, 0644);
   if (output_fh<0) {
     fprintf (stderr,_("cannot create output file '%s'.\n"),
-	    &file[1]);
+	    file);
     exit(1);
   }
 
-  fprintf (stderr,_("creating output file '%s'\n"), &file[1]);
+  fprintf (stderr,_("creating output file '%s'\n"), file);
 
   /* DNS lookup */
 
@@ -552,7 +640,7 @@ int main (int argc, char **argv) {
   sa.sin_family = hp->h_addrtype;
   /*sa.sin_port = 0x5000;*/ /* http port (80 => 50 Hex, switch Hi-/Lo-Word ) */
 
-  sa.sin_port = htons(1755) ; /* be2me_16(1755);  mms port 1755 */
+  sa.sin_port = htons(port) ; /* be2me_16(1755);  mms port 1755 */
 
   fprintf (stderr,_("port: %08x\n"), sa.sin_port);
 
@@ -658,9 +746,15 @@ int main (int argc, char **argv) {
 
 
   while (get_media_packet(s, packet_length)) {
-    fprintf(stderr,".");
-    /* nothing */
-
+    printf(".");
+    if (time != -1) {
+      time_t cur_time;
+      ::time(&cur_time);
+      if (cur_time > end_time) {
+        fprintf(stderr,"Maximum record time exceeded, stopping...\n");
+        break;
+      }
+    }
   }
 
   close (output_fh);
